import sdl2
import sdl2.ext
import sdl2.timer

import os
import time
import warnings

from aerforgesdl.input import Input
from aerforgesdl.color import Color
from aerforgesdl.sprite import Sprite
from aerforgesdl.entity import Entity

warnings.filterwarnings("ignore")

def init():
    sdl2.SDL_Init(sdl2.SDL_INIT_EVERYTHING)

class ForgeSDL:
    def __init__(self, width = 1200, height = 600, background_color = Color(20, 20, 20), vsync = True):
        self.width = width
        self.height = height
        self.background_color = background_color
        self.vsync = vsync

        self.objects = []

        sdl2.SDL_Init(sdl2.SDL_INIT_EVERYTHING)

        self.window = sdl2.SDL_CreateWindow("Forge".encode(), sdl2.SDL_WINDOWPOS_CENTERED, sdl2.SDL_WINDOWPOS_CENTERED, self.width, self.height, sdl2.SDL_WINDOW_SHOWN)
        
        if self.vsync:
            self.renderer = sdl2.SDL_CreateRenderer(self.window, -1, sdl2.SDL_RENDERER_ACCELERATED | sdl2.SDL_RENDERER_PRESENTVSYNC)

        else:
            self.renderer = sdl2.SDL_CreateRenderer(self.window, -1, 0)

        sdl2.SDL_SetRenderDrawBlendMode(self.renderer, sdl2.SDL_BLENDMODE_BLEND)

        self.path = os.path.dirname(os.path.abspath(__file__))

        icon = os.path.join(self.path, "./assets/icon/icon.png")
        icon = sdl2.ext.load_image(icon)
        sdl2.SDL_SetWindowIcon(self.window, icon)

        self.logo = os.path.join(self.path, "./assets/logo/logo.png")
        self.logo = Sprite(self, file = self.logo, width = 380, height = 80)
        self.logo.center()

        self.fade = Entity(self, width = self.width, height = self.height, color = Color(0, 0, 0))

        self.input = Input()

        self.destroyed = False

        self.start_time = time.time()

        self.dt = 1.0 / 60

        self.update_dt = 0
        self.update_handlers = []
        self.draw_handlers = []

        self.fps = 0

        self.keys = {
            "ESCAPE" : sdl2.SDL_SCANCODE_ESCAPE,

            "UP": sdl2.SDL_SCANCODE_UP, "DOWN": sdl2.SDL_SCANCODE_DOWN,
            "LEFT": sdl2.SDL_SCANCODE_LEFT, "RIGHT": sdl2.SDL_SCANCODE_RIGHT,

            "A": sdl2.SDL_SCANCODE_A, "B": sdl2.SDL_SCANCODE_B, "C": sdl2.SDL_SCANCODE_C,
            "D": sdl2.SDL_SCANCODE_D, "E": sdl2.SDL_SCANCODE_E, "F": sdl2.SDL_SCANCODE_F,
            "G": sdl2.SDL_SCANCODE_G, "H": sdl2.SDL_SCANCODE_H, "I": sdl2.SDL_SCANCODE_I,
            "J": sdl2.SDL_SCANCODE_J, "K": sdl2.SDL_SCANCODE_K, "L": sdl2.SDL_SCANCODE_L,
            "M": sdl2.SDL_SCANCODE_M, "N": sdl2.SDL_SCANCODE_N, "O": sdl2.SDL_SCANCODE_O,
            "P": sdl2.SDL_SCANCODE_P, "Q": sdl2.SDL_SCANCODE_Q, "R": sdl2.SDL_SCANCODE_R,
            "S": sdl2.SDL_SCANCODE_S, "T": sdl2.SDL_SCANCODE_T, "U": sdl2.SDL_SCANCODE_U,
            "V": sdl2.SDL_SCANCODE_V, "W": sdl2.SDL_SCANCODE_W, "X": sdl2.SDL_SCANCODE_X,
            "Y": sdl2.SDL_SCANCODE_Y, "Z": sdl2.SDL_SCANCODE_Z,
  
            "0": sdl2.SDL_SCANCODE_0, "1": sdl2.SDL_SCANCODE_1,"2": sdl2.SDL_SCANCODE_2, 
            "3": sdl2.SDL_SCANCODE_3, "4": sdl2.SDL_SCANCODE_4, "5": sdl2.SDL_SCANCODE_5,
            "6": sdl2.SDL_SCANCODE_6, "7": sdl2.SDL_SCANCODE_7, "8": sdl2.SDL_SCANCODE_8,
            "9": sdl2.SDL_SCANCODE_9,

            "F1": sdl2.SDL_SCANCODE_F1, "F2": sdl2.SDL_SCANCODE_F2, "F3": sdl2.SDL_SCANCODE_F3,
            "F4": sdl2.SDL_SCANCODE_F4, "F5": sdl2.SDL_SCANCODE_F5, "F6": sdl2.SDL_SCANCODE_F6,
            "F7": sdl2.SDL_SCANCODE_F7, "F8": sdl2.SDL_SCANCODE_F8, "F9": sdl2.SDL_SCANCODE_F9,
            "F10": sdl2.SDL_SCANCODE_F10, "F11": sdl2.SDL_SCANCODE_F11, "F12": sdl2.SDL_SCANCODE_F12,

            "LCTRL": sdl2.SDL_SCANCODE_LCTRL, "RCTRL": sdl2.SDL_SCANCODE_RCTRL,
            "LSHIFT": sdl2.SDL_SCANCODE_LSHIFT, "RSHIFT": sdl2.SDL_SCANCODE_RSHIFT,
            "LALT": sdl2.SDL_SCANCODE_LALT, "RALT": sdl2.SDL_SCANCODE_RALT,

            "TAB" : sdl2.SDL_SCANCODE_TAB, "CAPSLOCK" : sdl2.SDL_SCANCODE_CAPSLOCK,
            
            "HOME": sdl2.SDL_SCANCODE_HOME, "END": sdl2.SDL_SCANCODE_END,
            "INSERT": sdl2.SDL_SCANCODE_INSERT, "DELETE": sdl2.SDL_SCANCODE_DELETE,
            
            "RETURN": sdl2.SDL_SCANCODE_RETURN, "BACKSPACE": sdl2.SDL_SCANCODE_BACKSPACE,
            "SPACE": sdl2.SDL_SCANCODE_SPACE,
            
            "PLUS": sdl2.SDL_SCANCODE_KP_PLUS, "MINUS": sdl2.SDL_SCANCODE_KP_MINUS,
            "SLASH": sdl2.SDL_SCANCODE_SLASH, "BACKSLASH": sdl2.SDL_SCANCODE_BACKSLASH, 
            "ASTERISK": sdl2.SDL_SCANCODE_KP_MULTIPLY,
        }

        self.buttons = {
            "LEFT": sdl2.SDL_BUTTON_LEFT, "RIGHT": sdl2.SDL_BUTTON_RIGHT,
            "MIDDLE": sdl2.SDL_BUTTON_MIDDLE,
        }

    def delay(self, delay):
        sdl2.SDL_Delay(delay)

    def _update(self, dt):
        if not self.logo.destroyed:
            if self.start_time + 1 < time.time():
                if self.fade.get_alpha() != 0:
                    self.fade.set_alpha(self.fade.get_alpha() - 0.6)

                else:
                    self.fade.destroy()

                if self.start_time + 6 < time.time():
                    if self.logo.get_alpha() != 0:
                        self.logo.set_alpha(self.logo.get_alpha() - 1.2)

                    else:
                        self.logo.destroy()

        self.update_dt = self.update_dt + dt

        while self.update_dt > self.dt:
            for update in self.update_handlers:
                update(self.dt)

            self.update_dt = self.update_dt - self.dt

    def _draw(self):
        for draw in self.draw_handlers:
            draw()

        self.logo.draw()
        self.fade.draw()
        
    def run(self):
        sdl2.SDL_ShowWindow(self.window)

        prev_time = time.time()

        current = sdl2.SDL_GetPerformanceCounter()
        freq = sdl2.SDL_GetPerformanceFrequency()

        event = sdl2.SDL_Event()

        while self.destroyed == False:
            while sdl2.SDL_PollEvent(event) != 0:
                if event.type == sdl2.SDL_QUIT:
                    self.destroyed = True

                self.input.update(event)

            new = sdl2.SDL_GetPerformanceCounter()

            self._update((new - current) / freq)

            current = new

            sdl2.SDL_RenderClear(self.renderer)

            self._draw()

            sdl2.SDL_SetRenderDrawColor(self.renderer, self.background_color.r, self.background_color.g, self.background_color.b, self.background_color.a)
            sdl2.SDL_RenderPresent(self.renderer)

            try:
                self.fps = 1.0 / (time.time() - prev_time)

            except:
                pass
            
            prev_time = time.time()

        sdl2.SDL_DestroyRenderer(self.renderer)
        sdl2.SDL_DestroyWindow(self.window)
        sdl2.SDL_Quit()

    def draw(self, fn):
        self.draw_handlers.append(fn)

        return fn

    def update(self, fn):
        self.update_handlers.append(fn)

        return fn

    def drawall(self):
        for object in self.objects:
            object.draw()
        
    def updateall(self):
        for object in self.objects:
            object._update()

    def get_fps(self):
        return self.fps

    def destroy(self):
        self.destroyed = True